/*
 * Copyright (c) 2005, Regents of the University of California
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * * Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in
 *   the documentation and/or other materials provided with the
 *   distribution.  
 *
 * * Neither the name of the University of California, Berkeley nor
 *   the names of its contributors may be used to endorse or promote
 *   products derived from this software without specific prior 
 *   written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package blog;

/**
 * A random variable that serves as a node in a Bayes net (directed 
 * graphical model).  
 */
public interface BayesNetVar {
    /**
     * Returns true if the given world is complete enough to determine 
     * the value of this random variable.
     */
    boolean isDetermined(PartialWorld w);

    /**
     * Returns the conditional probability that this variable takes on 
     * the value that it has in the given world, given the values of 
     * its active parents.  
     *
     * @throws IllegalArgumentException if the given partial world is not 
     *                                  complete enough to determine the 
     *                                  values of this variable and its 
     *                                  active parents
     */
    double getProbOfValue(PartialWorld w);

    /**
     * Returns the value of this random variable in the given world.  
     *
     * @throws IllegalArgumentException if the given partial world is not 
     *                                  complete enough to determine the 
     *                                  value of this variable
     */
    Object getValue(PartialWorld w);	

    /**
     * Ensures that the given random variable is instantiated and
     * supported (that is, all its active ancestors are
     * instantiated) in the given world.  If new values are needed for 
     * variables, they are obtained by calling the <code>chooseValue</code> 
     * method on the given <code>ValueChooser</code> object.
     *
     * @return a ParentsAndValue object containing the active parent set 
     *         and (possibly new) value of this variable
     */
    ParentsAndValue ensureInstAndSupported(PartialWorld w, 
					   ValueChooser chooser);
}
